---
name: {{ name | tojson }}
description: {{ description | tojson }}
---

{{ instructions }}

{%- if scripts %}

---

# Scripts Disponíveis
{% for script in scripts %}
- `script_name`: `scripts/{{ script.file_name }}`
  - Quando usar: {{ script.when_to_use }}
  - Parâmetros:
    {%- if script.args %}
    {%- for arg in script.args %}
    - `{{ arg.name }}` ({{ arg.type }})
    {%- endfor %}
    {%- else %}
    Nenhum parâmetro.
    {%- endif %}
{% endfor %}
{%- endif %}
